#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2010 by VladX, <http://vladx.net>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, see <http://www.gnu.org/licenses/>.

import pygtk
pygtk.require('2.0')
import gtk
from io import BytesIO
try:
	from PIL import Image, ImageFilter, ImageOps
except ImportError:
	Image = None

class Effects:
	
	STROKE_WIDTH = 1
	STROKE_COLOR = 0x000000ff
	SHADOW_WIDTH = 10
	SHADOW_BLUR = 6
	SHADOW_COLOR = (0, 0, 0, 200)
	SHADOW_OFFSET = (0, 3)
	REFLECTION_HEIGHT = 0.4
	
	def __init__(self, pixbuf, effects):
		
		self.save = pixbuf.save
		self.save_to_callback = pixbuf.save_to_callback
		if not effects or not Image:
			return None
		self.src = pixbuf
		self.pilImage = False
		if 's' in effects and effects['s']:
			self.apply_stroke()
		if 'g' in effects and effects['g']:
			self.apply_grayscale()
		if 'd' in effects and effects['d']:
			self.apply_shadow()
		if 'r' in effects and effects['r']:
			self.apply_reflection()
	
	def _save_to_callback(self, callback, imageType):
		
		data = BytesIO()
		self.save(data, imageType)
		callback(data.getvalue())
		data.close()
	
	def pixbuf2array(self):
		
		s = 3
		w, h, rowstride = self.src.get_width(), self.src.get_height(), self.src.get_rowstride()
		array = []
		string = self.src.get_pixels()
		if self.src.get_has_alpha():
			s += 1
		if len(string) != rowstride * h:
			return False
		for i in xrange(h):
			for j in xrange(i * rowstride, i * rowstride + w * s, s):
				seq = ()
				for band in xrange(s):
					seq += (ord(string[j + band]),)
				array.append(seq)
		return array
	
	def apply_stroke(self):
		
		pixbuf = gtk.gdk.Pixbuf(self.src.get_colorspace(), self.src.get_has_alpha(), self.src.get_bits_per_sample(), self.src.get_width() + self.STROKE_WIDTH * 2, self.src.get_height() + self.STROKE_WIDTH * 2)
		pixbuf.fill(self.STROKE_COLOR)
		self.src.copy_area(0, 0, self.src.get_width(), self.src.get_height(), pixbuf, self.STROKE_WIDTH, self.STROKE_WIDTH)
		self.src = pixbuf
		self.save = self.src.save
		self.save_to_callback = self.src.save_to_callback
	
	def apply_shadow(self):
		
		if self.pilImage:
			width, height = self.pilImage.size
		else:
			width, height = self.src.get_width(), self.src.get_height()
			self.pilImage = Image.new('RGB', (width, height))
			self.pilImage.putdata(self.pixbuf2array())
		totalWidth, totalHeight = width + 2 * self.SHADOW_WIDTH + abs(self.SHADOW_OFFSET[0]), height + 2 * self.SHADOW_WIDTH + abs(self.SHADOW_OFFSET[1])
		shadowLeft, shadowTop = self.SHADOW_WIDTH + max(self.SHADOW_OFFSET[0], 0), self.SHADOW_WIDTH + max(self.SHADOW_OFFSET[1], 0)
		shadow = Image.new('RGBA', (totalWidth, totalHeight), (255, 255, 255, 0))
		shadow.paste(self.SHADOW_COLOR, (shadowLeft, shadowTop, shadowLeft + width, shadowTop + height))
		for i in xrange(self.SHADOW_BLUR):
			shadow = shadow.filter(ImageFilter.BLUR)
		shadow.paste(self.pilImage, (self.SHADOW_WIDTH - min(self.SHADOW_OFFSET[0], 0), self.SHADOW_WIDTH - min(self.SHADOW_OFFSET[1], 0)))
		self.pilImage = shadow
		self.save = self.pilImage.save
		self.save_to_callback = self._save_to_callback
	
	def apply_reflection(self):
		
		if self.pilImage:
			width, height = self.pilImage.size
		else:
			width, height = self.src.get_width(), self.src.get_height()
			self.pilImage = Image.new('RGB', (width, height))
			self.pilImage.putdata(self.pixbuf2array())
		gradientHeight = int(round(height * self.REFLECTION_HEIGHT))
		gradient = Image.new('L', (1, gradientHeight))
		for i in xrange(gradientHeight):
			gradient.putpixel((0, i), int(round(i * (200.0 / gradientHeight))))
		gradient = gradient.resize((width, gradientHeight))
		reflection = self.pilImage.crop((0, height - gradientHeight, width, height)).convert('RGBA')
		reflection = Image.composite(reflection, Image.new('RGBA', reflection.size, (255, 255, 255, 0)), gradient)
		reflection = reflection.transpose(Image.FLIP_TOP_BOTTOM)
		dst = Image.new('RGBA', (width, height + gradientHeight), (255, 255, 255, 0))
		dst.paste(self.pilImage, (0, 0))
		dst.paste(reflection, (0, height))
		self.pilImage = dst
		self.save = self.pilImage.save
		self.save_to_callback = self._save_to_callback
	
	def apply_grayscale(self):
		
		if not self.pilImage:
			width, height = self.src.get_width(), self.src.get_height()
			self.pilImage = Image.new('RGB', (width, height))
			self.pilImage.putdata(self.pixbuf2array())
		self.pilImage = ImageOps.grayscale(self.pilImage)
		self.pilImage.convert('L')
		self.save = self.pilImage.save
		self.save_to_callback = self._save_to_callback
